(* mspher.m *)

(* Version 2 *)

(* Created by Rolf Sulanke *)

(* Started March 20, 1995 , revised Sep 24,1998, last revision February 25,
 october 15, 2003. *)

(*This package contains procedures and functions for working in the Moebius
 geometry 
of spheres. It continues the package euvec.m which contains concepts of the
 euclidean 
geometry of spheres. Operations and functions of pseudo-euclidean vector algebra
 (see 
the package pseuvec.m and the notebook pseuklid.nb) are used. For applications
and further  procedures see the notebook mspheres.nb.*)

(* Free software: Feel free to apply, distribute or change the packages and
notebooks of the item "Spheres". Please, cite the source and notice the
changes, or new concepts,  which you introduced.*)

(*Remark: The functions psphere, paramsphere,showsphere are deleted now; replace
 them 
in an appropriate way by euklidsphere, euklidsphereplot3D. New functions:
 vradius,vcenter.*)
 
(* Changes 2003, Oct: spacevec renamed to spherevec. confinv renmed to
 coxeterinv. 
planev renamed to planevec. spunit is generalized; the more special old version
 is 
preserved. *)

BeginPackage["mspher`",{"euvec`", "pseuvec`"}]     



spt::usage ="spt[v] projects the vector v onto the vector
v/v[[dim]]-stb[dim] in the (dim-1)-space orthogonal to stb[dim].";

stepro::usage ="stepro is the stereographical projection in  pseudo-euclidean
terms (ind = 1), with values in the euclidean (dim - 2)-space.Warning:
stepro[v] is correct only for isotropic vectors v!";

invstepro::usage="invstepro is the inverse of the stereographic projection
 stepro (ind = 1).";


spunit::usage = "spunit[n][sList] is a parameter representation of the manifold
of all spacelike unit vectors in the n+1-dimensional pseudo-euclidean space of
index 1; the ranges are: 0<=s[[1]]<=2Pi, -Pi/2<=s[[j]]<=Pi/2 (1<j<n), s[[n]]
 arbitrary 
real; spunit[s,t,u,v]= spunit[4][{s,t,u,v}]."

pstg5frame::usage = "pstg5frame[s,t,u,v] is a pseudo-orthonormal moving frame
adapted to the manifold spunit[4][{s,t,u,v}] of spacelike unit vectors; the last
vector equals spunit[4][{s,t,u,v}]." 

vradius::usage = "vradius[vx] is the radius of the sphere in the euclidean
3-space corresponding to the spacelike vector vx of the pseudo-euclidean
 5-space, 
and Infinity if this sphere is a plane."

vcenter::usage = "vcenter[vx] is the center of the sphere in the euclidean 
3-space corresponding to the spacelike vector vx of the pseudo-euclidean
 5-space, 
and the infinite point if this sphere is a plane."

euklidsphere::usage = "euklidsphere[vx][u,v] is a parameter representation
of the sphere (resp. plane), with geographical parameters u,v, in the euclidean
3-space, which corresponds to the spacelike vector vx." 

euklidsphereplot3D::usage = "euklidsphereplot3D[vx,u0:-1,u1:1,v0:-1,v1:1,opts] 
is the plot of a region of the sphere corresponding to the spacelike vector 
vx; the region is defined by the parameters u0, u1, v0, v1. With 
the default values the whole sphere or a square with side length 2 of the plane 

is plotted; options opts are transferred to ParametricPlot3D."

sph4ptsvec::usage = "sph4ptsvec[p1, p2, p3, p4] yields a spacelike vector of the
 
pseudo-euclidean 5-space corresponding to the sphere through the four points
p1, p2, p3, p4 (in general position) of the euclidean 3-space.";

spherevec::usage = "spherevec[x, y, z, r] yields a spacelike vector of the
pseudo-euclidean 5-space corresponding to the sphere with center {x, y, z} and
radius
r of the euclidean 3-space.";

gencos::usage = "gencos[v,w] is the generalized cos of the (may be imginary) 
intersection angle of the spheres defined by the spacelike vectors v,w.";

coxeterinv::usage = "coxeterinv[r1, r2, d] is the conformal invariant of two
hyperspheres
 with radii r1, r2 and distance d of their centers.";

planevec::usage = "planevec[a,b,c,p] yields a spacelike vector of the
pseudo-euclidean  5-space defining the plane of the euclidean 3-space with
equation ax+by+cz=p.";

dimm = Global`dim;

indd = Global`ind;


Begin["`Private`"]


(*Projections of isotropic vectors on sphere points*)



spt[v_List] := Module[{w,u,d=Length[v]},w =v[[d]];
			If[w===0,  Return[Print[v ," has last component ",w]],
				u=Take[v/w,d-1];Return[u]]]
				
stepro[v_List] := sterproj[spt[v]]

(*The inverse  projection: sphere points - isotropic vector*)


invstepro[v_List] := Append[invsterproj[v],1]


(* Definition of the pseudosphere  in the pseudo-Euclidean n+1-Space: *)

spunit[n_][s_List] := Module[{ss,uu},
	If[Length[s] != n, Print["Incorrect arguments!"]; Return[]];
	ss = Take[s, n-1];
	uu = unit[n-1][ss];
	AppendTo[uu, 0];
	Return[uu*Cosh[s[[n]]] + stb[n+1,n+1]*Sinh[s[[n]]]]]

(* Case n = 4; the corresponding spheres in the Euclidean 3-space *)

spunit[s_,t_,u_,v_]:=
{Cos[s] Cos[t] Cos[u] Cosh[v],Cos[t] Cos[u] Cosh[v] Sin[s],
  Cos[u] Cosh[v] Sin[t],Cosh[v] Sin[u],Sinh[v]}
	
pstg5frame[s_, t_, u_, v_] :=
	{{- Sin[s],Cos[s],0,0,0},
		{-Cos[s] Sin[t],- Sin[s] Sin[t],Cos[t],0,0},
		{-Cos[s] Cos[t]  Sin[u],-Cos[t]  Sin[s] Sin[u],- Sin[t] Sin[u],Cos[u],0},
		{Cos[s] Cos[t] Cos[u] Sinh[v],Cos[t] Cos[u] Sin[s] Sinh[v],
      Cos[u] Sin[t] Sinh[v],Sin[u] Sinh[v],Cosh[v]},
		{Cos[s] Cos[t] Cos[u] Cosh[v],Cos[t] Cos[u] Cosh[v] Sin[s],
      Cos[u] Cosh[v] Sin[t],Cosh[v] Sin[u],Sinh[v]}}

vradius[wx_List] := Module[{s,q, vx,d= Length[wx]},
      q = pssp[wx,wx,d];
      If[q\[LessEqual] 0,Print[wx, " is not spacelike!"];Return[]];
      vx = wx/Sqrt[q];
      s=vx[[d]]- vx[[d-1]];
      If[s\[Equal]0, Return[Infinity],Return[1/s],Return[1/s]]]

vcenter[vx_List] := Module[{s,d = Length[vx]},
      If[pssp[vx,vx,d]\[LessEqual] 0,Print[vx, " is not spacelike!"];Return[]];
      s=vx[[d]]- vx[[d-1]];
      If[s\[Equal]0, Return[Table[Infinity,{d-2}]]];
      Return[Take[vx,d-2]/s]]

euklidsphere[va_List][u_,v_] := Module[{q},
      q = pssp[va,va];
      If[q\[LessEqual] 0, Print[va, " is not spacelike!"]; Return[]];
      w=va/Sqrt[q];
      If[w[[4]] == w[[5]], Goto[pla]];
      Return[sphere[vcenter[w],vradius[w]][u*Pi,v*Pi/2]];
      Label[pla];
       p = w[[5]];
      w = Take[w,3];
      ob = orthocomplement[{w}];
      AppendTo[ob, w];
      Return[ob[[3]]*p + ob[[1]]*u + ob[[2]]*v]
      ]/;Length[va]\[Equal] 5
    
euklidsphereplot3D[vn_List,u0_:-1,u1_:1, v0_:-1,v1_:1,opts___]:=
  ParametricPlot3D[Evaluate[euklidsphere[vn][u,v]],{u,u0,u1}, {v,v0,v1},opts]
    
(*Definition of a spacelike vector defining the sphere through four points of
 the 3e-space*)

sph4ptsvec[p1_List,p2_List,p3_List, p4_List] := Module[{v=null[5]},
		If[Length[p1]!=3||Length[p2]!=3||Length[p3]!=3||Length[p4]!=3,
		Print["An argument does not have the correct
dimension!"];Return[]];
v=Simplify[ 
        pscross[{invstepro[p1], invstepro[p2],invstepro[p3],invstepro[p4]}]];
		If[v===null[5],Print["The points are not in general
position!"];Return[],  Return[v], Return[v]]]

(*Definition of a spacelike vector defining the sphere with given center and
 radius *)

spherevec[x_, y_, z_, r_] := {2 x,  2 y, 2 z, -1 - r^2 + x^2 + y^2 + z^2, 1 -
 r^2
 + x^2 + y^2 + z^2}/(2 r) 

(*Definition of a spacelike vector defining the plane with equation ax+by+cz=p
 of the 3e-space*)

planevec[a_,b_,c_, p_] := If[Not[a^2+b^2+c^2===0],
	Return[{a, b, c, p, p}],Print["Normal vector null is not admissible!"];
    Return[]]


(*Invariants of a pair of spheres*)

gencos[v_List, w_List] := Simplify[pssp[v, w]/Sqrt[pssp[v,v] pssp[w,w]]]

coxeterinv[r1_, r2_, d_] := (r1^2 + r2^2 -d^2)/(2 r1 r2)

End[]

Protect[spt, stepro, invstepro, spunit,euklidsphere,
 euklidsphereplot3D, 
pstg5frame, sph4ptsvec, spherevec, gencos, coxeterinv, planevec, vradius,
 vcenter]

EndPackage[]
